<?php

namespace Drupal\ld_json_schema\Form;

use Drupal\Core\Entity\EntityForm;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Component\Serialization\Json;

/**
 * Class JsonSchemaEntityForm. Provides the form to manage JSON entity.
 */
class JsonSchemaEntityForm extends EntityForm {

  /**
   * {@inheritdoc}
   */
  public function form(array $form, FormStateInterface $form_state) {
    $form = parent::form($form, $form_state);

    $json_schema_entity = $this->entity;
    $form['label'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Url'),
      '#maxlength' => 255,
      '#default_value' => $json_schema_entity->label(),
      '#description' => $this->t("Url for the Json schema entity."),
      '#required' => TRUE,
    ];

    $form['id'] = [
      '#type' => 'machine_name',
      '#default_value' => $json_schema_entity->id(),
      '#machine_name' => [
        'exists' => '\Drupal\ld_json_schema\Entity\JsonSchemaEntity::load',
      ],
      '#disabled' => !$json_schema_entity->isNew(),
    ];

    $form['json'] = [
      '#type' => 'textarea',
      '#title' => $this->t('JSON'),
      '#default_value' => $json_schema_entity->json(),
      '#description' => $this->t("Raw JSON for the Json schema entity."),
      '#required' => TRUE,
    ];

    /* You will need additional form elements for your custom properties. */

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function validateForm(array &$form, FormStateInterface $form_state) {

    if ($form_state->getValue('label')[0] != "/") {
      $form_state->setErrorByName('label', \Drupal::translation()->translate('URI should start with / symbol.'));
    }

    if (!Json::decode($form_state->getValue('json'))) {
      $form_state->setErrorByName('json', \Drupal::translation()->translate('JSON field value should be a valid JSON.'));
    }
  }

  /**
   * {@inheritdoc}
   */
  public function save(array $form, FormStateInterface $form_state) {
    $json_schema_entity = $this->entity;
    $status = $json_schema_entity->save();

    switch ($status) {
      case SAVED_NEW:
        $this->messenger()->addMessage($this->t('Created the %label Json schema entity.', [
          '%label' => $json_schema_entity->label(),
        ]));
        break;

      default:
        $this->messenger()->addMessage($this->t('Saved the %label Json schema entity.', [
          '%label' => $json_schema_entity->label(),
        ]));
    }
    $form_state->setRedirectUrl($json_schema_entity->toUrl('collection'));
  }

}
