<?php

namespace Drupal\ld_json_schema\Plugin\metatag\Tag;

use Drupal\Component\Serialization\Json;
use Drupal\Core\Form\FormStateInterface;
use Drupal\metatag\Plugin\metatag\Tag\Description;

/**
 * Provides a plugin for the 'ld_json_schema_jsonld_raw' meta tag.
 *
 * @MetatagTag(
 *   id = "ld_json_schema_jsonld_raw",
 *   label = @Translation("Raw JSON"),
 *   description = @Translation("Raw JSON for JSON-LD schema."),
 *   name = "ld_json_schema_jsonld_raw",
 *   group = "ld_json_schema_jsonld",
 *   weight = 90,
 *   type = "label",
 *   secure = FALSE,
 *   multiple = FALSE,
 *   long = TRUE,
 * )
 */
class JsonLdRaw extends Description {

  /**
   * {@inheritdoc}
   */
  public function form(array $element = []): array {
    $form = parent::form($element);
    unset($form['#maxlength']);
    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function output(): array {
    $element = [
      '#tag' => 'script',
      '#attributes' => ['type' => 'application/ld+json'],
      '#value' => $this->value(),
      '#weight' => 100,
    ];

    return $element;
  }

  /**
   * {@inheritdoc}
   */
  public static function validateTag(array &$element, FormStateInterface $form_state): void {
    parent::validateTag($element, $form_state);

    if ($element['#value'] && !Json::decode($element['#value'])) {
      $form_state->setError($element, \Drupal::translation()->translate('Raw JSON field value should be a valid JSON.'));
    }
  }

}
